package vnsim.gui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileSystemView;

import sun.misc.URLClassPath;

import com.sun.javaws.jnl.JARDesc;
import com.sun.javaws.jnl.LaunchDesc;
import com.sun.javaws.jnl.ResourcesDesc;
import com.sun.jnlp.JNLPClassLoader;


/**
 * An utility class with several methods that are used throughout the GUI classes
 * @author cipsm
 *
 */
public class Utils {

	private static final Utils _instance = new Utils();

	public static Utils getInstance() {
		return _instance;
	}

	private Utils() { }

	public FileSystemView getSystemView(String root) {
		return new ClassSystemView(root);
	}

	/** A FileSystemView implementation that is used to display the classes loaded by the GUI used ClassLoader.. */
	public static class ClassSystemView extends FileSystemView {

		private String root;
		private SystemViewHandler handler = null;

		public ClassSystemView(String root) {
			this.root = root;
			System.out.println(root);
			// init the context
			try {
				Method m = ClassLoader.class.getDeclaredMethod("getBootstrapClassPath", new Class[0]);
				m.setAccessible(true);
				URLClassPath p = (URLClassPath)m.invoke(getClass().getClassLoader(), new Object[0]);
				URL u[] = p.getURLs();
				for (int i=0; i<u.length; i++) {
					String path = u[i].toString();
					if (path.startsWith("file://"))
						path = path.substring(7);
					else if (path.startsWith("file:/"))
						path = path.substring(6);
					if(File.separatorChar == '\\') {
						path = path.replace("/", "\\");
					}
					path = path.replace("%20", " ").trim(); // for compatibility of space with the FS
					if (path.endsWith(".jar")) {
//						System.out.println(path);
						JarHandler h = new JarHandler(path);
						final File ff[] = h.listResources(root);
						if (ff != null && ff.length != 0) {
							handler = h;
							break;
						}
					} else {
						FileHandler h = new FileHandler(path);
						final File ff[] = h.listResources(root);
						if (ff != null && ff.length != 0) {
							handler = h;
							break;
						}
					}
				}
				if (handler == null) {
					String classpath = System.getProperty("java.class.path");
//					System.out.println("classpath="+classpath);
					
					if (classpath != null) {
						String pp[] = null;
						if (File.separatorChar == '\\')
							pp = classpath.split(";");
						else
							pp = classpath.split(":");
						if (pp != null && pp.length != 0) {
							for (int i=0; i<pp.length; i++) {
								String path = pp[i];
								if (path.startsWith("file://"))
									path = path.substring(7);
								else if (path.startsWith("file:/"))
									path = path.substring(6);
								path = path.replace("%20", " "); // for compatibility of space with the FS
								if (path.endsWith(".jar")) {
//									System.out.println(path);
									JarHandler h = new JarHandler(path);
									final File ff[] = h.listResources(root);
									if (ff != null && ff.length != 0) {
										handler = h;
										break;
									}
								} else {
									FileHandler h = new FileHandler(path);
									final File ff[] = h.listResources(root);
									if (ff != null && ff.length != 0) {
										handler = h;
										break;
									}
								} 
							}
						}
					}
				}
				if (handler == null) { // still ?
					ClassLoader l = getClass().getClassLoader();
//					System.out.println("am intrat aici "+l);
					if (l instanceof URLClassLoader) {
						URLClassLoader lc = (URLClassLoader)l;
						u = lc.getURLs();
//						System.out.println(u.length);
						for (int i=0; i<u.length; i++) {
							String path = u[i].toString();
//							System.out.println(path);
							if (path.startsWith("file://"))
								path = path.substring(7);
							else if (path.startsWith("file:/"))
								path = path.substring(6);
							if(File.separatorChar == '\\') {
								path = path.replace("/", "\\");
							}
							path = path.replace("%20", " ").trim(); // for compatibility of space with the FS
							if (path.endsWith(".jar")) {
//								System.out.println(path);
								JarHandler h = new JarHandler(path);
								final File ff[] = h.listResources(root);
								if (ff != null && ff.length != 0) {
									handler = h;
									break;
								}
							} else {
								FileHandler h = new FileHandler(path);
								final File ff[] = h.listResources(root);
								if (ff != null && ff.length != 0) {
									handler = h;
									break;
								}
							}
						}
					} else {
						try {
							if (l instanceof JNLPClassLoader) {
								JNLPClassLoader jl = (JNLPClassLoader)l;
								LaunchDesc ld = jl.getLaunchDesc();
								final List<JARDesc> jarDescs = new ArrayList<JARDesc>();
								ResourcesDesc rd = ld.getResources();
								if (rd != null) {
									JARDesc[] jars = rd.getEagerOrAllJarDescs(true);
									for (int i = 0; i < jars.length; i++)
										if (jars[i].isJavaFile() && !jars[i].isNativeLib())
											jarDescs.add(jars[i]);
								}
								for (JARDesc jd : jarDescs) {
									URL location = jd.getLocation();
									if (!location.getFile().endsWith("maps.jar")) continue;
									// download it ?
									File dirToDownload = new File(System.getProperty("user.home")+File.separatorChar+"maps");
									if (!dirToDownload.exists()) {
										dirToDownload.mkdirs();
									}
									String path = System.getProperty("user.home")+File.separatorChar+"maps"+File.separatorChar+"maps.jar";
									File tmp = new File(path);  
									InputStream isDown;
									URLConnection connection = location.openConnection();
									//write from web to local file
									FileOutputStream fOS = new FileOutputStream(tmp);
									isDown = connection.getInputStream();
									int nBufferSize = 1024;
									byte buff[] = new byte[nBufferSize];
									int nRead = -1;
									do {
										nRead = isDown.read(buff);
										if ( nRead > 0 ) {
											fOS.write(buff, 0, nRead);
										}
									} while ( nRead > 0 );
									fOS.flush();
									fOS.close();
									isDown.close();
//									System.out.println(path);
									JarHandler h = new JarHandler(path);
									final File ff[] = h.listResources(root);
									if (ff != null && ff.length != 0) {
										handler = h;
										break;
									}
								}
							}
						} catch (Throwable t1) { 
							// can happen due to different versions of JavaWS
						}
						if (handler == null) {
							FileHandler h = new FileHandler(".");
							if (h.listResources(root) != null)
								handler = h;
						}
					}
				}
				if (handler == null) {
					FileHandler h = new FileHandler(".");
					if (h.listResources(root) != null)
						handler = h;
				}
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}

		public File[] getRoots() {
			if (handler == null)
				return null;
			return new File[] { new CustomFile(root, true) };
		}

		public boolean isRoot(File f) {
			if (f == null) return false;
			return f.getName().equals(root);
		}

		public File[] getFiles(File dir, boolean useFileHiding) {
//			System.out.println(dir.getName()+" "+(dir instanceof CustomFile));
			if (handler != null) {
				return handler.listResources(dir.getName());
			} 
			return null;
		}

		public String[] list() {
			if (handler != null) {
				File f[] = handler.listResources(root);
				if (f == null || f.length == 0) {
					return null;
				}
				String[] s = new String[f.length];
				for (int i=0; i<s.length; i++) s[i] = f[i].getName();
				return s;
			}
			return null;
		}

		public File getHomeDirectory() {
			if (handler == null) return null;
			return new CustomFile(root, true);
		}

		public File getDefaultDirectory() {
			if (handler == null) return super.getDefaultDirectory();
			return new CustomFile(root, true);
		}

		/** Never used in this case */
		public File createNewFolder(File containingDir) throws IOException {
			return null;
		}

	}

	static class CustomFile extends File {
		private String fileName;
		private boolean isDir;
		public CustomFile(String fileName, boolean isDir) {
			super(fileName);
			this.fileName = fileName;
			this.isDir = isDir;
		}

		public boolean isDirectory() {
			return isDir;
		}

		public String getName() {
//			try {
//			throw new Exception();
//			} catch (Exception e) {
//			e.printStackTrace();
//			}
			return fileName;
		}

		public String getParent() {
//			System.out.println(super.getParent());
			return "";
		}
	}

	/** generic interface used to declare the common methods for handling a dir or jar file */
	interface SystemViewHandler {
		public File[] listResources(String resource);
	}

	/** Utility class used to handle the searching of particular resources inside a jar file */
	static class JarHandler implements SystemViewHandler {
		private String jarName;
		public JarHandler(String jarName) {
			this.jarName = jarName;
		}
		public File[] listResources(String resource) {
			if (resource == null) return null;
			File f = new File(jarName);
			try {
				if (f.isFile() && f.getName().endsWith(".jar")){
					final JarFile jf = new JarFile(f);
//					System.out.println(f);
					final Enumeration<JarEntry> e = jf.entries();
					JarEntry je;
					Vector<File> v = new Vector<File>();
					while (e.hasMoreElements()){
						je = e.nextElement();
						String name = je.getName();
						if (!name.startsWith("/")) name = "/"+name;
//						System.out.println(name + " " + resource);
						if (name.startsWith(resource) && name.length() > resource.length()) {
							String end = name.substring(resource.length());
							if (end.length() > 0 && end.startsWith("/")) end = end.substring(1);
							if (end.contains("/") || end.length() < 1) continue; // uninteresting result
							v.add(new CustomFile(end, je.isDirectory()));
						} else continue;
					}
					if (v.size() == 0) return null;
					CustomFile ff[] = new CustomFile[v.size()];
					ff = v.toArray(ff);
					return ff;
				}
			} catch (Throwable t) {
				t.printStackTrace();
				return null;
			}
			return null;
		}
	}

	static class FileHandler implements SystemViewHandler { 
		private String rootDir;
		public FileHandler(String dirName) {
			this.rootDir = dirName;
		}
		public File[] listResources(String resource) {
			if(File.separatorChar == '\\') {
				resource = resource.replace("/", "\\").trim();
			}
			File dir = new File(rootDir+File.separatorChar+resource);
			if (dir.exists() && dir.isDirectory()) {
				File f[] = dir.listFiles();
				if (f == null || f.length == 0) {
					return f;
				}
				CustomFile cf[] = new CustomFile[f.length];
				for (int i=0; i<f.length; i++)
					cf[i] = new CustomFile(f[i].getName(), f[i].isDirectory());
				return cf;
			} 
			else {
			System.out.println("Dir "+rootDir+File.separatorChar+resource+" does not exist");
			}
			return new File[0];
		}
	}

	// method used to get the lock on a inputstream of a an internal file representation...
	public InputStream openStream(String name) throws Exception {
//		System.out.println(name);
		URL u = getClass().getClassLoader().getResource(name);

		try {
			return u.openConnection().getInputStream();
		} catch (Throwable t1) {
				
			try {
					if(File.separatorChar == '\\') {
						name = name.replace("/", "\\").trim();
					}
					return new FileInputStream(System.getProperty("user.dir") + name);
				}catch (Throwable t2) {
				
					try {
						if(File.separatorChar == '\\') {
							name = name.replace("/", "\\").trim();
						}
					
						return new FileInputStream(System.getProperty("user.home") + name);
					} catch(Throwable t3) {
					
						File f = new File(name);
						if (!f.exists() || f.isDirectory()) return null;
						return new FileInputStream(name);
					}

				}
		}

	}

	public OutputStream openWriteStream(String name) throws Exception {
		if (name.startsWith("/")) name = name.substring(1);
		URL u = getClass().getClassLoader().getResource(name);
		try {
			return u.openConnection().getOutputStream();
		} catch (Throwable t) {
			try {
				if(File.separatorChar == '\\') {
					name = name.replace("/", "\\").trim();
				}
				return new FileOutputStream(System.getProperty("user.dir") + File.separatorChar + name);
			} catch (Throwable t2) {
				return new FileOutputStream(name);
			}
		} 
	}

	public static void main(String args[]) {
		FileSystemView v = Utils.getInstance().getSystemView("/maps/variations");
		File f[] = v.getFiles(v.getRoots()[0], false);
		JFileChooser fc=new JFileChooser("/maps/variations", v);
		fc.setDialogTitle("Choose the file specifying the flow variations");
		int retVal=fc.showOpenDialog(null);
		System.out.println(fc.getSelectedFile().getAbsolutePath());
	}

} // end of class Utils

