/************************************************************************************
 * Copyright (C) 2008 by Politehnica University of Bucharest and Rutgers University
 * All rights reserved.
 * Refer to LICENSE for terms and conditions of use.
 ***********************************************************************************/
package vnsim.map.object;




import java.util.ArrayList;

import vnsim.vehicular.simulator.*;
/**
 * @author Victor-Radu
 *
 */
public class Road implements Comparable<Road>, java.io.Serializable {
	
	/** <code>serialVersionUID</code> */
	private static final long serialVersionUID = -5883719321862303634L;
	
	protected long id;
	protected String type;					//Ave., St. or other
	protected String name;
	protected byte[] directionPrefix; 
	protected byte[] directionSuffix;
	public ArrayList<Point> points = new ArrayList<Point>();	// sorted vector of points that are maximum Globals.MAXSEGLEN from each other
	public ArrayList<Cross> crosses = new ArrayList<Cross>();	// crosses wtih other roads
	
	protected int nextRdIdx = -1,			// index of next segment along a road with the same name 
				prevRdIdx = -1;				// index of previous ...
											// both used just in the map building process

	protected byte[] roadinfo;				// A41, A31 or other (see TIGER files specs)

	public SortedCarVector carsOnThisRoad=new SortedCarVector();
	
	//added for simulation
	public int laneNo;

	public Road(){

	}
	
	public void printRoad(boolean printPoints) {
		System.out.println("( " + id + " " + new String(directionPrefix) + " " + name + " " +
				new String(directionSuffix) + " " + type + " " + new String(roadinfo) + " " + points.size() + ")");
		if (printPoints)
			for (int i=0; i < points.size();i++){
				System.out.print("\t"+(Point)points.get(i));
				if (i<points.size()-1){
					System.out.println(((Point)points.get(i)).distanceTo((Point)points.get(i+1)));
				}
			}
	}

	public int compareTo(Road r) {
		return (int)(id - r.id);
	}

	public ArrayList getCrosses() {
		return crosses;
	}

	public void setCrosses(ArrayList crosses) {
		this.crosses = crosses;
	}
	public byte[] getDirectionPrefix() {
		return directionPrefix;
	}
	public void setDirectionPrefix(byte[] directionPrefix) {
		this.directionPrefix = directionPrefix;
	}
	public byte[] getDirectionSuffix() {
		return directionSuffix;
	}
	public void setDirectionSuffix(byte[] directionSuffix) {
		this.directionSuffix = directionSuffix;
	}
	public long getId() {
		return id;
	}
	public void setId(long id) {
		this.id = id;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public int getNextRdIdx() {
		return nextRdIdx;
	}
	public void setNextRdIdx(int nextRdIdx) {
		this.nextRdIdx = nextRdIdx;
	}
	public ArrayList getPoints() {
		return points;
	}
	public void setPoints(ArrayList points) {
		this.points = points;
	}
	public int getPrevRdIdx() {
		return prevRdIdx;
	}
	public void setPrevRdIdx(int prevRdIdx) {
		this.prevRdIdx = prevRdIdx;
	}
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}
	
	public byte[] getRoadinfo() {
		return roadinfo;
	}
	public void setRoadinfo(byte[] roadinfo) {
		this.roadinfo = roadinfo;
	}

	public int oneWay; // 0=two-way ; 
					   // 1=one-way from point 0 to last point
					   // 2=one-way from last point to point 0
}
