/************************************************************************************
 * Copyright (C) 2008 by Politehnica University of Bucharest and Rutgers University
 * All rights reserved.
 * Refer to LICENSE for terms and conditions of use.
 ***********************************************************************************/
package vnsim.network.dsrc;

import org.jfree.ui.ApplicationFrame;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.Spring;
import javax.swing.SpringLayout;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JLabel;
import org.jfree.chart.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.xy.*;
import org.jfree.data.xy.*;
import org.jfree.data.time.*;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.imagemap.*;
import org.jfree.chart.labels.CustomXYToolTipGenerator;
import org.jfree.chart.urls.XYURLGenerator; 
import java.util.*;
import java.text.SimpleDateFormat;

public class DSRCChart extends ApplicationFrame
{
	public DSRCChart(final String title,final ArrayList<DSRCStatisticComponent> results)
	{
		super(title);
		
		SpringLayout l = new SpringLayout();
		setLayout(l);
		SpringLayout.Constraints constraint;
		
		XYSeries s1=new XYSeries("OK");
		XYSeries s2=new XYSeries("Collision");
		XYSeries s3=new XYSeries("Corrupted");
		XYSeries s4=new XYSeries("Lost RX");
		XYSeries s5=new XYSeries("Lost TX");
		XYSeries s6=new XYSeries("Lost Per");
		XYSeries s17=new XYSeries("Lost Weak");
		
		s1.add(0,0);
		s2.add(0,0);
		s3.add(0,0);
		s4.add(0,0);
		s5.add(0,0);
		s6.add(0,0);
		s17.add(0,0);
		for(int i=1;i<results.size();i++)
		{
			s1.add(i,(double)100*results.get(i).getOkPackets()/results.get(i).getTotalPackets());
			s2.add(i,(double)100*results.get(i).getCollisionPackets()/results.get(i).getTotalPackets());
			s3.add(i,(double)100*results.get(i).getCorruptedPackets()/results.get(i).getTotalPackets());
			s4.add(i,(double)100*results.get(i).getLostRxPackets()/results.get(i).getTotalPackets());
			s5.add(i,(double)100*results.get(i).getLostTxPackets()/results.get(i).getTotalPackets());
			s6.add(i,(double)100*results.get(i).getLostPerPackets()/results.get(i).getTotalPackets());
			s17.add(i,(double)100*results.get(i).getWeakPackets()/results.get(i).getTotalPackets());
		}
		
		XYSeriesCollection dataset = new XYSeriesCollection();
        dataset.addSeries(s1);
        dataset.addSeries(s2);
        dataset.addSeries(s3);
        dataset.addSeries(s4);
        dataset.addSeries(s5);
        dataset.addSeries(s6);
        dataset.addSeries(s17);
       
        final JFreeChart chart = ChartFactory.createXYLineChart(
                "Received Packets lost/ok",          // chart title
                "Category",               // domain axis label
                "Value",                  // range axis label
                dataset,                  // data
                PlotOrientation.VERTICAL,
                true,                     // include legend
                true,
                false
            );
        
        final XYPlot plot = chart.getXYPlot();
        final NumberAxis domainAxis = new NumberAxis("Simulation minute");
        final NumberAxis rangeAxis = new NumberAxis("Percentage of total packets/sim minute");
        plot.setDomainAxis(domainAxis);
        plot.setRangeAxis(rangeAxis);
        
        /*
        try {
      	  ChartUtilities.saveChartAsJPEG(
                new java.io.File("Test.jpg"), chart, 500, 300);
      	} catch (java.io.IOException exc) {
      	    System.err.println("Error writing image to file");
      	}
      	*/
        
        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        //setContentPane(chartPanel);
        
        constraint = l.getConstraints(chartPanel);
		constraint.setX(Spring.constant(0));
		constraint.setY(Spring.constant(0));
		constraint.setWidth(Spring.constant(500));
		constraint.setHeight(Spring.constant(270));
        add(chartPanel);
        
        
        XYSeries s7=new XYSeries("FV");
		XYSeries s8=new XYSeries("NFV");
		XYSeries s9=new XYSeries("LAV");
		XYSeries s10=new XYSeries("RAV");
		
		for(int i=0;i<results.size();i++)
		{
			s7.add(i,results.get(i).getReceivedFromFV());
			s8.add(i,results.get(i).getReceivedFromNFV());
			s9.add(i,results.get(i).getReceivedFromLAV());
			s10.add(i,results.get(i).getReceivedFromRAV());
		}
		
		XYSeriesCollection dataset2 = new XYSeriesCollection();
        dataset2.addSeries(s7);
        dataset2.addSeries(s8);
        dataset2.addSeries(s9);
        dataset2.addSeries(s10);
        
        final JFreeChart chart2 = ChartFactory.createXYLineChart(
                "Received Packets by transmitter",          // chart title
                "Category",               // domain axis label
                "Value",                  // range axis label
                dataset2,                  // data
                PlotOrientation.VERTICAL,
                true,                     // include legend
                true,
                false
            );
        
        final XYPlot plot2 = chart2.getXYPlot();
        final NumberAxis domainAxis2 = new NumberAxis("Simulation time(min)");
        final NumberAxis rangeAxis2 = new NumberAxis("Number of packets/sim minute");
        plot2.setDomainAxis(domainAxis2);
        plot2.setRangeAxis(rangeAxis2);
        
        final ChartPanel chartPanel2 = new ChartPanel(chart2);
        chartPanel2.setPreferredSize(new java.awt.Dimension(500, 270));
        
        constraint = l.getConstraints(chartPanel2);
		constraint.setX(Spring.constant(500));
		constraint.setY(Spring.constant(0));
		constraint.setWidth(Spring.constant(500));
		constraint.setHeight(Spring.constant(270));
        add(chartPanel2);
        
        
        XYSeries s11=new XYSeries("FV");
		XYSeries s12=new XYSeries("NFV");
		XYSeries s13=new XYSeries("LAV");
		XYSeries s14=new XYSeries("RAV");
		
		for(int i=0;i<results.size();i++)
		{
			s11.add(i,results.get(i).getIRTFromFV());
			s12.add(i,results.get(i).getIRTFromNFV());
			s13.add(i,results.get(i).getIRTFromLAV());
			s14.add(i,results.get(i).getIRTFromRAV());
		}
		
		XYSeriesCollection dataset3 = new XYSeriesCollection();
        dataset3.addSeries(s11);
        dataset3.addSeries(s12);
        dataset3.addSeries(s13);
        dataset3.addSeries(s14);
        
        final JFreeChart chart3 = ChartFactory.createXYLineChart(
                "IRT by transmitter",          // chart title
                "Category",               // domain axis label
                "Value",                  // range axis label
                dataset3,                  // data
                PlotOrientation.VERTICAL,
                true,                     // include legend
                true,
                false
            );
        
        final XYPlot plot3 = chart3.getXYPlot();
        final NumberAxis domainAxis3 = new NumberAxis("Simulation time (min)");
        final NumberAxis rangeAxis3 = new NumberAxis("IRT (sec)/sim minute");
        plot3.setDomainAxis(domainAxis3);
        plot3.setRangeAxis(rangeAxis3);
        
        final ChartPanel chartPanel3 = new ChartPanel(chart3);
        chartPanel3.setPreferredSize(new java.awt.Dimension(500, 270));
        
        constraint = l.getConstraints(chartPanel3);
		constraint.setX(Spring.constant(0));
		constraint.setY(Spring.constant(270));
		constraint.setWidth(Spring.constant(500));
		constraint.setHeight(Spring.constant(270));
        add(chartPanel3);
        
        
        XYSeries s15=new XYSeries("Emergency Message Recv");
        XYSeries s16=new XYSeries("Emergency Message Sent");
        
        for(int i=0;i<results.size();i++)
		{
			s15.add(i,results.get(i).getEmergencyReceived());
			s16.add(i,results.get(i).getEmergencySent());
		}
        
        XYSeriesCollection dataset4 = new XYSeriesCollection();
        dataset4.addSeries(s15);
        dataset4.addSeries(s16);
        
        final JFreeChart chart4 = ChartFactory.createXYLineChart(
                "Emergency Messages",          // chart title
                "Category",               // domain axis label
                "Value",                  // range axis label
                dataset4,                  // data
                PlotOrientation.VERTICAL,
                true,                     // include legend
                true,
                false
            );
        
        final XYPlot plot4 = chart4.getXYPlot();
        final NumberAxis domainAxis4 = new NumberAxis("Simulation time (min)");
        final NumberAxis rangeAxis4 = new NumberAxis("Number of messages/sim minute");
        plot4.setDomainAxis(domainAxis4);
        plot4.setRangeAxis(rangeAxis4);
        
        final ChartPanel chartPanel4 = new ChartPanel(chart4);
        chartPanel4.setPreferredSize(new java.awt.Dimension(500, 270));
        
        constraint = l.getConstraints(chartPanel4);
		constraint.setX(Spring.constant(500));
		constraint.setY(Spring.constant(270));
		constraint.setWidth(Spring.constant(500));
		constraint.setHeight(Spring.constant(270));
        add(chartPanel4);
        
        //number of cars per every simulation minute
        
        if(results.size()!=0)
        {	
	        String[] columnNames= { "Sim minute","No of cars"};
	        Object data[][]=new Object[results.size()][2];
	        
	        for(int j=0;j<results.size();j++)
	        {
	        	data[j][0]=j;
	        	data[j][1]=new Integer(results.get(j).getNumberOfCars());
	        }
	        
	        
	        final JTable carsTable=new JTable(data,columnNames);
	        JScrollPane scrollPane = new JScrollPane(carsTable);
	        
	        constraint=l.getConstraints(scrollPane);
	        constraint.setX(Spring.constant(1020));
			constraint.setY(Spring.constant(0));
			constraint.setWidth(Spring.constant(180));
			constraint.setHeight(Spring.constant(550));
	        add(scrollPane);
        }
        
        JButton close=new JButton("Close");
        
        constraint=l.getConstraints(close);
        constraint.setX(Spring.constant(10));
		constraint.setY(Spring.constant(550));
		constraint.setWidth(Spring.constant(70));
		constraint.setHeight(Spring.constant(30));
        
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				dispose();
				System.exit(0);
			}
		});
		
		add(close);
		
		final JLabel written=new JLabel("Statistical data written to file");
		
		constraint=l.getConstraints(written);
        constraint.setX(Spring.constant(220));
		constraint.setY(Spring.constant(550));
		constraint.setWidth(Spring.constant(350));
		constraint.setHeight(Spring.constant(30));
		written.setVisible(false);
		
		
		
		JButton writeToFile=new JButton("Write to file");
		
		constraint=l.getConstraints(writeToFile);
        constraint.setX(Spring.constant(90));
		constraint.setY(Spring.constant(550));
		constraint.setWidth(Spring.constant(120));
		constraint.setHeight(Spring.constant(30));
		
		
		add(written);
		writeToFile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				
				
					try{
						ExcelWriter exwriter=new ExcelWriter();
						
						Calendar cal=Calendar.getInstance();
						SimpleDateFormat formatter=new SimpleDateFormat("dd_MM_yyyy");
						String time=formatter.format(cal.getTime());

						Calendar cal2=Calendar.getInstance();
						SimpleDateFormat formatter2=new SimpleDateFormat("HH_mm_ss");
						String date=formatter2.format(cal2.getTime());
						
						exwriter.write(date+"_"+time+".xls",results);
						written.setText("Statistical data written to file "+date+"_"+time+".xls");
						written.setVisible(true);
					}
					catch(Exception e){System.out.println(e.getMessage());}
		        
			    
			}
		});
		
		add(writeToFile);
	}
	
}