/************************************************************************************
 * Copyright (C) 2008 by Politehnica University of Bucharest and Rutgers University
 * All rights reserved.
 * Refer to LICENSE for terms and conditions of use.
 ***********************************************************************************/
package vnsim.network.dsrc;

import vnsim.map.object.Globals;

public class DSRCStatisticComponent
{
	private int componentId;
	
	//cummulative number of packets received from a specific transmitter
	private long receivedFromFV,receivedFromNFV,receivedFromLAV,receivedFromRAV;
	
	//packet average inter reception time for different transmitters
	private double IRTFromFV,IRTFromNFV,IRTFromLAV,IRTFromRAV;
	
	//packet success probability from a given transmitter
	private double PSPFromFV ,PSPFromNFV,PSPFromLAV,PSPFromRAV;
	
	public long weakPackets;
	public long totalPackets;
	public long corruptedPackets;
	public long collisionPackets;
	public long lostTxPackets;
	public long lostRxPackets;
	public long lostPerPackets;
	public long OkPackets;
	
//	send failure packets
	public long sendFailureRXPackets=0;
	public long sendFailureNoisePackets=0;
	
	public int numberOfCars;
	
	public long emergencyReceived=0;
	public long emergencySent=0;
	
	public DSRCStatisticComponent(int componentId,long weakPackets,long totalPackets,long corruptedPackets,long collisionPackets,long lostTxPackets
			,long lostRxPackets,long lostPerPackets,long OkPackets,long sendFailureRXPackets,long sendFailureNoisePackets,
			long receivedFromFV,long receivedFromNFV,long receivedFromLAV,long receivedFromRAV,double IRTFromFV,double IRTFromNFV,
			double IRTFromLAV,double IRTFromRAV,double PSPFromFV,double PSPFromNFV,double PSPFromLAV,double PSPFromRAV,int numberOfCars,long emergencyReceived,long emergencySent)
	{
		this.weakPackets=weakPackets;
		this.totalPackets=totalPackets;
		this.corruptedPackets=corruptedPackets;
		this.collisionPackets=collisionPackets;
		this.lostTxPackets=lostTxPackets;
		this.lostRxPackets=lostRxPackets;
		this.lostPerPackets=lostPerPackets;
		this.OkPackets=OkPackets;
		
		this.sendFailureNoisePackets=sendFailureNoisePackets;
		this.sendFailureRXPackets=sendFailureRXPackets;
		
		this.receivedFromFV=receivedFromFV;
		this.receivedFromLAV=receivedFromLAV;
		this.receivedFromNFV=receivedFromNFV;
		this.receivedFromRAV=receivedFromRAV;
		
		this.IRTFromFV=((receivedFromFV!=0)?IRTFromFV/this.receivedFromFV:0)/Globals.SECOND;
		this.IRTFromLAV=((receivedFromLAV!=0)?IRTFromLAV/this.receivedFromLAV:0)/Globals.SECOND;
		this.IRTFromNFV=((receivedFromNFV!=0)?IRTFromNFV/this.receivedFromNFV:0)/Globals.SECOND;
		this.IRTFromRAV=((receivedFromRAV!=0)?IRTFromRAV/this.receivedFromRAV:0)/Globals.SECOND;
		
		this.PSPFromFV=PSPFromFV;
		this.PSPFromLAV=PSPFromLAV;
		this.PSPFromNFV=PSPFromNFV;
		this.PSPFromRAV=PSPFromRAV;
		
		this.componentId=componentId;
		
		this.numberOfCars=numberOfCars;
		
		this.emergencyReceived=emergencyReceived;
		this.emergencySent=emergencySent;
		
	}
	
	public long getTotalPackets()
	{
		return this.totalPackets;
	}
	
	public long getCorruptedPackets()
	{
		return this.corruptedPackets;
	}
	
	public long getCollisionPackets()
	{
		return this.collisionPackets;
	}
	
	public long getLostTxPackets()
	{
		return this.lostTxPackets;
	}
	
	public long getLostRxPackets()
	{
		return this.lostRxPackets;
	}
	
	public long getLostPerPackets()
	{
		return this.lostPerPackets;
	}
	
	public long getOkPackets()
	{
		return this.OkPackets;
	}
	
	public long getSendFailureRXPackets()
	{
		return this.sendFailureRXPackets;
	}
	
	public long getSendFailureNoisePackets()
	{
		return this.sendFailureNoisePackets;
	}
	
	public long getReceivedFromFV()
	{
		return this.receivedFromFV;
	}
	
	public long getReceivedFromNFV()
	{
		return this.receivedFromNFV;
	}
	
	public long getReceivedFromLAV()
	{
		return this.receivedFromLAV;
	}
	
	public long getReceivedFromRAV()
	{
		return this.receivedFromRAV;
	}
	
	public double getIRTFromFV()
	{
		return this.IRTFromFV;
	}
	
	public double getIRTFromNFV()
	{
		return this.IRTFromNFV;
	}
	
	public double getIRTFromLAV()
	{
		return this.IRTFromLAV;
	}
	
	public double getIRTFromRAV()
	{
		return this.IRTFromRAV;
	}
	
	public double getPSPFromFV()
	{
		return this.PSPFromFV;
	}
	
	public double getPSPFromNFV()
	{
		return this.PSPFromNFV;
	}
	
	public double getPSPFromLAV()
	{
		return this.PSPFromLAV;
	}
	
	public double getPSPFromRAV()
	{
		return this.PSPFromRAV;
	}
	
	public int getComponentId()
	{
		return this.componentId;
	}
	
	public int getNumberOfCars()
	{
		return this.numberOfCars;
	}
	
	public long getEmergencyReceived()
	{
		return this.emergencyReceived;
	}
	
	public long getEmergencySent()
	{
		return this.emergencySent;
	}
	
	public long getWeakPackets()
	{
		return this.weakPackets;
	}
}