/************************************************************************************
 * Copyright (C) 2008 by Politehnica University of Bucharest and Rutgers University
 * All rights reserved.
 * Refer to LICENSE for terms and conditions of use.
 ***********************************************************************************/
package vnsim.network.dsrc;

import java.io.*;
import org.dom4j.*;
import java.util.HashMap;
import java.util.ArrayList;

public class ExcelWriter
{
private File outputFile = null;
    
    /** Creates a new instance of JavaExcel */
    public ExcelWriter() {
    }
    
    //This function writes the SpreadsheetML document to a file
    public void write(String file,ArrayList<DSRCStatisticComponent> results) throws Exception {
        outputFile = new File(file);
        Document doc = DocumentHelper.createDocument();
        HashMap pmap = new HashMap();
        pmap.put("progid","Excel.Sheet");
        doc.addProcessingInstruction("mso-application",pmap);
        doc.add(this.createElements(results));
        FileWriter out = new FileWriter(outputFile);
        doc.write(out);
        out.flush();
        out.close();
    }
    
    //This function creates the root of the SpreadsheetML and
    //populates it with necesary elements.
    private Element createElements(ArrayList<DSRCStatisticComponent> results) throws Exception {
        //Create all Elements
        Element workbook = this.createWorkbook();
        Element worksheet = this.createWorksheet("Simualtion Data");
        workbook.add(worksheet);
        
        Element table = this.createTable();
        worksheet.add(table);
        Element row1 = this.createRow("1");
        table.add(row1);
        Element cellA1 = this.createCell();
        row1.add(cellA1);
        Element dataA1 = this.createData("String","Received Packets lost/ok");
        cellA1.add(dataA1);
        
        Element row2 = this.createRow("2");
        table.add(row2);
        Element cell=this.createCell();
    	row2.add(cell);
    	Element data = this.createData("String","Ok packets");
        cell.add(data);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row2.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getOkPackets()));
            cellaux.add(dataaux);
        }
        
        Element row3 = this.createRow("3");
        table.add(row3);
        Element cell3=this.createCell();
    	row3.add(cell3);
    	Element data3 = this.createData("String","Collision Packets");
        cell3.add(data3);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row3.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getCollisionPackets()));
            cellaux.add(dataaux);
        }
        
        Element row4 = this.createRow("4");
        table.add(row4);
        Element cell4=this.createCell();
    	row4.add(cell4);
    	Element data4 = this.createData("String","Corrupted Packets");
        cell4.add(data4);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row4.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getCorruptedPackets()));
            cellaux.add(dataaux);
        }
        
        Element row5 = this.createRow("5");
        table.add(row5);
        Element cell5=this.createCell();
    	row5.add(cell5);
    	Element data5 = this.createData("String","Lost RX Packets");
        cell5.add(data5);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row5.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getLostRxPackets()));
            cellaux.add(dataaux);
        }
        
        Element row6 = this.createRow("6");
        table.add(row6);
        Element cell6=this.createCell();
    	row6.add(cell6);
    	Element data6 = this.createData("String","Lost TX Packets");
        cell6.add(data6);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row6.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getLostTxPackets()));
            cellaux.add(dataaux);
        }
        
        Element row7 = this.createRow("7");
        table.add(row7);
        Element cell7=this.createCell();
    	row7.add(cell7);
    	Element data7 = this.createData("String","Lost PER Packets");
        cell7.add(data7);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row7.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getLostPerPackets()));
            cellaux.add(dataaux);
        }
        
        Element row8 = this.createRow("8");
        table.add(row8);
        Element cell8=this.createCell();
    	row8.add(cell8);
    	Element data8 = this.createData("String","Lost Weak Packets");
        cell8.add(data8);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row8.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getWeakPackets()));
            cellaux.add(dataaux);
        }
        
        Element row9 = this.createRow("9");
        table.add(row9);
        Element cell9=this.createCell();
    	row9.add(cell9);
    	Element data9 = this.createData("String","Received Packets by transmitter");
        cell9.add(data9);
        
        Element row10 = this.createRow("10");
        table.add(row10);
        Element cell10=this.createCell();
    	row10.add(cell10);
    	Element data10 = this.createData("String","FV");
        cell10.add(data10);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row10.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getReceivedFromFV()));
            cellaux.add(dataaux);
        }
        
        Element row11 = this.createRow("11");
        table.add(row11);
        Element cell11=this.createCell();
    	row11.add(cell11);
    	Element data11 = this.createData("String","NFV");
        cell11.add(data11);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row11.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getReceivedFromNFV()));
            cellaux.add(dataaux);
        }
        
        Element row12 = this.createRow("12");
        table.add(row12);
        Element cell12=this.createCell();
    	row12.add(cell12);
    	Element data12 = this.createData("String","LAV");
        cell12.add(data12);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row12.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getReceivedFromLAV()));
            cellaux.add(dataaux);
        }
        
        Element row13 = this.createRow("13");
        table.add(row13);
        Element cell13=this.createCell();
    	row13.add(cell13);
    	Element data13 = this.createData("String","RAV");
        cell13.add(data13);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row13.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getReceivedFromRAV()));
            cellaux.add(dataaux);
        }
        
        Element row14 = this.createRow("14");
        table.add(row14);
        Element cell14=this.createCell();
    	row14.add(cell14);
    	Element data14 = this.createData("String","IRT by transmitter");
        cell14.add(data14);
        
        Element row15 = this.createRow("15");
        table.add(row15);
        Element cell15=this.createCell();
    	row15.add(cell15);
    	Element data15 = this.createData("String","FV");
        cell15.add(data15);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row15.add(cellaux);
        	Element dataaux = this.createData("Number",Double.toString(results.get(i).getIRTFromFV()));
            cellaux.add(dataaux);
        }
        
        Element row16 = this.createRow("16");
        table.add(row16);
        Element cell16=this.createCell();
    	row16.add(cell16);
    	Element data16 = this.createData("String","NFV");
        cell16.add(data16);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row16.add(cellaux);
        	Element dataaux = this.createData("Number",Double.toString(results.get(i).getIRTFromNFV()));
            cellaux.add(dataaux);
        }
        
        Element row17 = this.createRow("17");
        table.add(row17);
        Element cell17=this.createCell();
    	row17.add(cell17);
    	Element data17 = this.createData("String","LAV");
        cell17.add(data17);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row17.add(cellaux);
        	Element dataaux = this.createData("Number",Double.toString(results.get(i).getIRTFromLAV()));
            cellaux.add(dataaux);
        }
        
        Element row18 = this.createRow("18");
        table.add(row18);
        Element cell18=this.createCell();
    	row18.add(cell18);
    	Element data18 = this.createData("String","RAV");
        cell18.add(data18);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row18.add(cellaux);
        	Element dataaux = this.createData("Number",Double.toString(results.get(i).getIRTFromRAV()));
            cellaux.add(dataaux);
        }
        
        Element row19 = this.createRow("19");
        table.add(row19);
        Element cell19=this.createCell();
    	row19.add(cell19);
    	Element data19 = this.createData("String","Emergency Messages");
        cell19.add(data19);
        
        Element row20 = this.createRow("20");
        table.add(row20);
        Element cell20=this.createCell();
    	row20.add(cell20);
    	Element data20 = this.createData("String","Received");
        cell20.add(data20);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row20.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getEmergencyReceived()));
            cellaux.add(dataaux);
        }
        
        Element row21 = this.createRow("21");
        table.add(row21);
        Element cell21=this.createCell();
    	row21.add(cell21);
    	Element data21 = this.createData("String","Sent");
        cell21.add(data21);
        for(int i=1;i<results.size();i++)
        {
            Element cellaux=this.createCell();
        	row21.add(cellaux);
        	Element dataaux = this.createData("Number",Long.toString(results.get(i).getEmergencySent()));
            cellaux.add(dataaux);
        }
        return workbook;
    }
    
    private Element createWorksheet(String name) {
        Element e = DocumentHelper.createElement("Worksheet");
        e.addAttribute("ss:Name", name);
        e.setQName(new QName("Worksheet",new Namespace(null,"urn:schemas-microsoft-com:office:spreadsheet")));
        return e;
    }
    
    private Element createTable() {
        return DocumentHelper.createElement("ss:Table");
    }
    
    private Element createRow(String index) {
        Element e = DocumentHelper.createElement("ss:Row");
        e.addAttribute("ss:Index",index);
        return e;
    }
    
    private Element createCell() {
        return DocumentHelper.createElement("ss:Cell");
    }
    
    private Element createData(String type, String data) {
        Element e = DocumentHelper.createElement("ss:Data");
        e.addAttribute("ss:Type",type);
        e.setText(data);
        return e;
    }
    
    private Element createWorkbook() throws Exception {
        Element root =  DocumentHelper.createElement("Workbook");
        //Set up the necessary namespaces
        root.add(new Namespace("x","urn:schemas-microsoft-com:office:excel"));
        root.add(new Namespace("ss","urn:schemas-microsoft-com:office:spreadsheet"));
        root.setQName(new QName("Workbook",new Namespace(null,"urn:schemas-microsoft-com:office:spreadsheet")));
        return root;
    }
    
}