/************************************************************************************
 * Copyright (C) 2008 by Politehnica University of Bucharest and Rutgers University
 * All rights reserved.
 * Refer to LICENSE for terms and conditions of use.
 ***********************************************************************************/
//Petroaca - two ray propagation model
package vnsim.network.propagation;

import vnsim.map.object.Globals;
import vnsim.network.propagation.FreeSpace;

public class TwoRay
{
	private static double Hr=Globals.AH;
	private static double Ht=Globals.AH;
	private static double crossoverDistance=(4*Math.PI*Ht*Hr)/Globals.DSRC_LAMBDA;
	
	public static double getPr(double Pt,double R)
	{
		if(Pt<=0.0 || R<=0.0)
			return -1;
		
		if(R<=crossoverDistance)
		{
			return FreeSpace.getPr(Pt,R);
		}
		else
		{
			double aux=Math.pow(Ht,2)*Math.pow(Hr,2);
			double R4=Math.pow(R,4);
			
			return (Pt*Math.pow(Globals.ANTENNA_GAIN,2)*aux)/(R4*Globals.SYSTEM_LOSS);
		}
	}
	
	public static double getPrAlt(double Pt,double R)
	{
		if(Pt<=0 || R<=0)
			return -1;
		
		double pathloss = -(2*Globals.ANTENNA_GAIN);
		double freeSpace = (4.0 * Math.PI * R) / Globals.DSRC_LAMBDA;
		double planeEarth = (R * R) / (Hr*Ht);

		if (planeEarth > freeSpace){
			pathloss += 20.0 * Math.log10(planeEarth);
		}else{
			pathloss += 20.0 * Math.log10(freeSpace);
		}
	    
		return Pt-pathloss;
	}
}