package vnsim.socialNet;

import java.util.ArrayList;
import java.util.Random;

import vnsim.vehicular.routePlan.RouteComputingUtils;
import vnsim.vehicular.simulator.Location;
import vnsim.vehicular.simulator.RouteSegment;
import vnsim.vehicular.scenarios.Route;

public class Mobility {
	public ArrayList<ArrayList<Integer>> groups;

	public CellInformation cells[][];
	
	public double sideLengthX;
	public double sideLengthY;
	
	public ArrayList<NodeInformation> ni;
	
	public Mobility(){
		groups = new ArrayList<ArrayList<Integer>>();
		cells = new CellInformation[GlobalNetwork.ROWS][GlobalNetwork.COLUMNS];
		
		sideLengthX=GlobalNetwork.LENGTHX/GlobalNetwork.COLUMNS;
		sideLengthY=GlobalNetwork.LENGTHY/GlobalNetwork.ROWS;
		
		System.out.println("side length X "+sideLengthX);
		System.out.println("side length Y "+sideLengthY);
		
		double lat = GlobalNetwork.pointMin.getLatitude();
		double lon = GlobalNetwork.pointMin.getLongitude();
		
		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				cells[i][j] = new CellInformation();
				cells[i][j].setMinPoint(lon+(double)j*sideLengthX,lat+(double)i*sideLengthY);
				cells[i][j].setMaxPoint(lon+(double)(j+1)*sideLengthX,lat+(double)(i+1)*sideLengthY);;	
			}
		}
		
		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				if(Util.intersecRoad(cells[i][j])==true)
					cells[i][j].enable();
				else
					cells[i][j].disable();
			}
		}
		
		ni = new ArrayList<NodeInformation>();
		
		for(int i=0;i<GlobalNetwork.nrHosts;i++){
			ni.add(new NodeInformation(i,GlobalNetwork.nrHosts));
		}
	}
	
	public void setPriorityEntry(){
		Random rand = new Random();
		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				if(cells[i][j].isAvailable()==false){
					cells[i][j].setPriorityEntry(0);
				}else{
					cells[i][j].setPriorityEntry(rand.nextInt(10)+1);
				}
			}
		}
	}
	
	public void setPriorityExit(){
		Random rand = new Random();
		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				int r = rand.nextInt(10)+1;
				while(r==cells[i][j].priorityEntry){
					r = rand.nextInt(10)+1;
				}
				if(cells[i][j].isAvailable()==false){
					cells[i][j].setPriorityExit(0);
				}else{
					cells[i][j].setPriorityExit(r);
				}
			}
		}
	}
	
	public void setPriorityEntryCenter(boolean center){
		Random rand = new Random();
		int valcenter = 0;
		int valedge = 0;

		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				if(center==false){
					valcenter = rand.nextInt(5)+1;
					valedge = rand.nextInt(5)+5;
				}else{
					valcenter = rand.nextInt(5)+5;
					valedge = rand.nextInt(5)+1;
				}
				if(cells[i][j].isAvailable()==false){
					cells[i][j].setPriorityEntry(0);
				}else{
					if((i>=GlobalNetwork.ROWS/3)&&(i<=2*GlobalNetwork.ROWS/3)){
						if((j>=GlobalNetwork.COLUMNS/3)&&(j<=2*GlobalNetwork.COLUMNS/3))
							cells[i][j].setPriorityEntry(valcenter);
					}else{
						cells[i][j].setPriorityEntry(valedge);
					}
				}
			}
		}
	}
	
	public void setPriorityExitCenter(boolean center){
		Random rand = new Random();
		int valcenter = 0;
		int valedge = 0;

		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				if(center==false){
					valcenter = rand.nextInt(5)+1;
					valedge = rand.nextInt(5)+5;
				}else{
					valcenter = rand.nextInt(5)+5;
					valedge = rand.nextInt(5)+1;
				}
				if(cells[i][j].isAvailable()==false){
					cells[i][j].setPriorityExit(0);
				}else{
					if((i>=GlobalNetwork.ROWS/3)&&(i<=2*GlobalNetwork.ROWS/3)){
						if((j>=GlobalNetwork.COLUMNS/3)&&(j<=2*GlobalNetwork.COLUMNS/3))
							cells[i][j].setPriorityExit(valcenter);
					}else{
						cells[i][j].setPriorityExit(valedge);
					}
				}
			}
		}
	}
	
	public void print_cell(){
		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				System.out.println("[ij] "+i+" "+j+" Min "+cells[i][j].pointMin.getLatitude()+" "+cells[i][j].pointMin.getLongitude());
				System.out.println("[ij] "+i+" "+j+" Max "+cells[i][j].pointMax.getLatitude()+" "+cells[i][j].pointMax.getLongitude());
				System.out.println("[ij] "+i+" "+j+" ok to use = "+cells[i][j].isAvailable());
				System.out.println("[ij] "+i+" "+j+" entrie = "+cells[i][j].priorityEntry);
				System.out.println("[ij] "+i+" "+j+" exit = "+cells[i][j].priorityExit);
			}
		}
	}
	
	public void print_groups(){
		System.out.println("Grupuri");
		for(int i=0;i<groups.size();i++){
			for(int j=0;j<groups.get(i).size();j++){
				System.out.print(" "+groups.get(i).get(j));
			}
			System.out.println();
		}
	}
	
	public void print_adjacency(){
		int i,j;
		for(i=0;i<ni.size();i++){
			for(j=0;j<GlobalNetwork.nrHosts;j++){
				System.out.print(" "+ni.get(i).getAdjacency(j));
			}
			System.out.println();
		}
	}
	
	public void print_interaction(){
		int i,j;
		for(i=0;i<ni.size();i++){
			for(j=0;j<GlobalNetwork.nrHosts;j++){
				System.out.print(" "+ni.get(i).getIneraction(j));
			}
			System.out.println();
		}
	}
	
	public void createInteraction(int nrHosts,int nrGroups){

		int numberOfMembers[] = new int[nrGroups];
		int groups[][] = new int[nrHosts][nrHosts];
		
		for (int i=0;i<nrGroups;i++) {
			numberOfMembers[i]=0;
		}
		
		for(int i=0;i<nrHosts;i++){
			for(int j=0;j<nrHosts;j++){
				groups[i][j]=0;
			}
		}
		
		for (int i=0;i<nrHosts;i++) {
			int groupId=i%nrGroups;
			groups[groupId][numberOfMembers[groupId]]=i+1;
			numberOfMembers[groupId]+=1;	
		}
		
		/*for(int i=0;i<ni.size();i++){
			for(int j=0;j<GlobalNetwork.nrHosts;j++){
				System.out.print(" "+groups[i][j]);
			}
			System.out.println();
		}*/
		
		int seed = (int)GlobalNetwork.SEED;
		int th = (int)(100*GlobalNetwork.THESHOLD);
		Random generator;
		if(seed != 0){
			generator = new Random(seed);
		}
		else{
			generator = new Random();
		}
		
		for(int i=0;i<nrHosts;i++){
			for (int j=0;j<nrHosts;j++) {
				 if (Util.areInAGroup(i+1,j+1,groups,nrGroups,numberOfMembers)==true) {
					double rand = 1.0-(generator.nextInt(th)/100.0);
					ni.get(i).setIneraction(j, rand);
				 } else{ //the hosts are not in the same group
					 double rand = generator.nextInt(th)/100.0;
					 ni.get(i).setIneraction(j, rand);
					}
			}
		}
		
		//print_interaction();
		
		for(int i=0;i<nrHosts;i++){
			for (int j=0;j<nrHosts;j++) {
				if (Util.areInAGroup(i+1,j+1,groups,nrGroups,numberOfMembers)==true) {
					double rand1 = generator.nextInt((int)10*th)/100.0;
					if ((rand1)<GlobalNetwork.REWIRE && i!=j){
						boolean found=false;
						for (int z=0;z<nrHosts;z++){
							if (Util.areInAGroup(i+1,z+1,groups,nrGroups,numberOfMembers)==false){
									if((ni.get(i).getIneraction(z)<GlobalNetwork.THESHOLD)&&(found==false)){
										//System.out.println("final z "+z+" i "+i+" j "+j+" rand1 "+rand1);
										double rand = 1.0-(generator.nextInt(th)/100.0);
										ni.get(i).setIneraction(z, rand);
										ni.get(z).setIneraction(i, 1.0-(generator.nextInt(th)/100.0));
										found=true;
									}
							}
						}
						if(found==true){
							//System.out.println("ds "+i+" "+j);
							double rand = generator.nextInt(th)/100.0;
							ni.get(i).setIneraction(j, rand);
							ni.get(j).setIneraction(i, generator.nextInt(th)/100.0);
						}
					}
				}
			}
		}
	}


	public void deleteGroups(){
		groups = new ArrayList<ArrayList<Integer>>();
	}
	
	public void reconfigureEntrie(){
		
		int nrHosts = GlobalNetwork.nrHosts;
		int nrGroups = (int)GlobalNetwork.GROUPS;
		
		createInteraction(nrHosts,nrGroups);
		
		for(int i=0;i<nrHosts;i++){
			ni.get(i).generateAdjacency();
		}		

		//print_interaction();
		//print_adjacency();

		double previousModth=0.0;
		double modth=0.1;
		
		do {
			double betw[] = new double[nrHosts];
			
			for (int i=0;i<nrHosts; i++)
				betw[i]=0;
			
			Util.calculate_betweenness(betw,ni,nrHosts);
				
			/*System.out.println("Betweeness");
			for(int i=0;i<betw.length;i++){
				System.out.print(betw[i]+" ");
			}*/
		
	
			deleteGroups();
			
			int numberOfGroups=Util.getGroups(ni, groups ,nrHosts);
				
			//System.out.println("Avem "+numberOfGroups+" grupuri");

			previousModth=modth;
	
			modth=Util.splitNetwork_Threshold(ni, betw, nrHosts, modth);
		
		}
		while ((previousModth<modth)&&(modth>-1));
		
		//print_adjacency();
		print_groups();
		GlobalNetwork.GROUPS = groups.size();
	}
	
	public boolean existsPriority(int r, int priority){
		for (int i=0;i<GlobalNetwork.ROWS;i++){
			for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
				if(priority == 0){
					if(cells[i][j].priorityEntry == r)
						return true;
				}
				else{
					if(cells[i][j].priorityExit == r)
						return true;
				}
			}
		}
		return false;
	}
	
	public void placeNodes(int priority){
		Random rand;
		
		//print_cell();
		
		if(GlobalNetwork.SEED == 0){
			rand = new Random();
		}
		else{
			rand = new Random();
		}
		
		for (int i=0;i<groups.size();i++) {
			int cellIdX=rand.nextInt(GlobalNetwork.ROWS);
			int cellIdY=rand.nextInt(GlobalNetwork.COLUMNS);
			
			if(GlobalNetwork.PRIORITY == true){
				int r = rand.nextInt(10)+1;
				while(!existsPriority(r,priority)){
					r = rand.nextInt(10);
				}
				//System.out.println("Grup "+i+" prior "+r);
				
				if(priority==0){
					while((!cells[cellIdX][cellIdY].isAvailable()) || (cells[cellIdX][cellIdY].priorityEntry<r)){
						cellIdX=rand.nextInt(GlobalNetwork.ROWS);
						cellIdY=rand.nextInt(GlobalNetwork.COLUMNS);
					}
					//System.out.println("Winner entrie:"+cellIdX+" "+cellIdY+" prior "+cells[cellIdX][cellIdY].priorityEntry);
				}else{
					while((!cells[cellIdX][cellIdY].isAvailable()) || (cells[cellIdX][cellIdY].priorityExit<r)){
						cellIdX=rand.nextInt(GlobalNetwork.ROWS);
						cellIdY=rand.nextInt(GlobalNetwork.COLUMNS);
					}
					//System.out.println("Winner exit:"+cellIdX+" "+cellIdY+" prior "+cells[cellIdX][cellIdY].priorityExit);
				}
			}
			else{
				while(!cells[cellIdX][cellIdY].isAvailable()){
					cellIdX=rand.nextInt(GlobalNetwork.ROWS);
					cellIdY=rand.nextInt(GlobalNetwork.COLUMNS);
				}
			}
			
			System.out.println("pentru grupul "+i+" cell x "+cellIdX+" y "+cellIdY);

			for (int j=0;j<groups.get(i).size();j++) {
				int hostId=groups.get(i).get(j);
				ni.get(hostId-1).cellX=cellIdX+1;
				ni.get(hostId-1).cellY=cellIdY+1;

				cells[cellIdX][cellIdY].numberOfHosts+=1;
			}
		}
	}
	
	
	public void initalPositions(){
		
		Random rand;
		if(GlobalNetwork.SEED == 0){
			rand = new Random();
		}
		else{
			rand = new Random();
		}
	
		for (int k=0;k<GlobalNetwork.nrHosts;k++) {	
			int index = rand.nextInt(cells[ni.get(k).cellX-1][ni.get(k).cellY-1].getNoPoints());
			ni.get(k).addStartLocation(cells[ni.get(k).cellX-1][ni.get(k).cellY-1].getValidLocation(index));	
			ni.get(k).addStartPoint(cells[ni.get(k).cellX-1][ni.get(k).cellY-1].getValidPoint(index));
		}
/*		
		for (int i=0;i<GlobalNetwork.nrHosts;i++) {		
			System.out.println("nodul "+i+" set X "+ni.get(i).start.getLongitude()+" set Y "+ni.get(i).start.getLatitude()+" in patratul ("+ni.get(i).cellX+" "+ni.get(i).cellY+")");				  	  
		}
		*/
	}
	
	public void deleteAll(){
		groups = new ArrayList<ArrayList<Integer>>();
		
		ni = new ArrayList<NodeInformation>();
		
		for(int i=0;i<GlobalNetwork.nrHosts;i++){
			ni.add(new NodeInformation(i,GlobalNetwork.nrHosts));
		}
	}
	
	public void reconfigureExit(){
		int nrHosts = GlobalNetwork.nrHosts;
		int nrGroups = (int)GlobalNetwork.GROUPS;
		
		createInteraction(nrHosts,nrGroups);
		
		for(int i=0;i<nrHosts;i++){
			ni.get(i).generateAdjacency();
		}		

		int numberOfGroups=Util.getGroups(ni, groups ,nrHosts);
		
		createInteraction(nrHosts,nrGroups);
		
		for(int i=0;i<nrHosts;i++){
			ni.get(i).generateAdjacency();
		}
		
		//print_adjacency();
		print_groups();
	}
	
	public ArrayList<Location> getLocations(){
		ArrayList<Location> list = new ArrayList<Location>();
		
		for(int i=0;i<ni.size();i++){
			list.add(ni.get(i).getLocation());
		}
		
		return list;
	}
	
	public void getPositions(int cellX[], int cellY[]){
		for(int i=0;i<ni.size();i++){
			cellX[i] = ni.get(i).cellX-1;
			cellY[i] = ni.get(i).cellY-1;
		}
	}
	
	ArrayList<Location> SegmentsRoute;
	
	public int[] findCA(int xinit, int xfinal, int yinit, int yfinal,int CA[][]){
		int CAmax = 0;
		int xgasit=-1,ygasit=-1;
		for(int i=xinit;i<xfinal;i++){
			for(int j=yinit;j<yfinal;j++){
				if(CA[i][j]>CAmax){
					CAmax = CA[i][j];
					xgasit = i;
					ygasit = j;
				}
			}
		}
		
		int ret[] = new int[3];
		ret[0]= 0;
		if((CAmax==0)){
			ret[0]=1;
		}else
		if(!cells[xgasit][ygasit].isAvailable()){
			ret[0]=1;
		}	
		ret[1] = xgasit;
		ret[2] = ygasit;
	//	System.out.println("in find"+xinit+" "+yinit+" "+xfinal+" "+yfinal+" "+xgasit+" "+ygasit);

		if(xgasit==xfinal && ygasit==yfinal)
			ret[0]=1;
		return ret;
	}
	
	public void findPath(int xinit, int xfinal, int yinit, int yfinal,int CA[][]){
		int gasit[]=null;
		System.out.println(xinit+" "+yinit+" "+xfinal+" "+yfinal);
		if(xfinal<xinit && yfinal<yinit){
			gasit = findCA(xfinal,xinit,yfinal,yinit,CA);
		}
		if(xfinal<xinit && yfinal>yinit){
			gasit = findCA(xfinal,xinit,yinit,yfinal,CA);
		}
		if(xfinal>xinit && yfinal<yinit){
			gasit = findCA(xinit,xfinal,yfinal,yinit,CA);
		}
		if(xfinal>xinit && yfinal>yinit){
			gasit = findCA(xinit,xfinal,yinit,yfinal,CA);
		}
		if(gasit!=null){
			int xgasit,ygasit;
			boolean recursiv = true;
			xgasit = gasit[1];
			ygasit = gasit[2];
			if(gasit[0]==1){ //we have an error
				xgasit = xfinal;
				ygasit = yfinal;
				recursiv = false;
			}
			if(xgasit==xinit && ygasit==yinit){
				xgasit = xfinal;
				ygasit = yfinal;
				recursiv = false;
			}
			if(xgasit==xfinal && ygasit==yfinal){
				recursiv = false;
			}
			Location loc;
			Random rand = new Random();
		//	System.out.println(cells[xgasit][ygasit].isAvailable());
			int index = rand.nextInt(cells[xgasit][ygasit].getNoPoints());
			loc = cells[xgasit][ygasit].getValidLocation(index);
			SegmentsRoute.add(loc);
			
			if(recursiv)
				findPath(xgasit,xfinal,ygasit,yfinal,CA);
		}
	}
	
	public ArrayList<RouteSegment> getSocialAtractionRoute(int i,int cellX[], int cellY[]){
		int CA[][] = new int[GlobalNetwork.ROWS][GlobalNetwork.COLUMNS];

		for(int j=0;j<GlobalNetwork.ROWS;j++){
			for(int k=0;k<GlobalNetwork.COLUMNS;k++){
				CA[j][k] = 0;
			}
		}
		for(int j=0;j<GlobalNetwork.nrHosts;j++){
			if(ni.get(i).getAdjacency(j)==1){
				CA[cellX[j]][cellY[j]] += 1;
			}
		}

		int xfinal = ni.get(i).cellX-1;
		int yfinal = ni.get(i).cellY-1;
		CA[xfinal][yfinal]=1;
		System.out.println("Pentru nodul "+i);
		/*	for (int k=0;k<GlobalNetwork.ROWS;k++){
				for (int j=0;j<GlobalNetwork.COLUMNS;j++) {
					System.out.print(" "+CA[k][j]);
				}
				System.out.println();
			}*/
		SegmentsRoute = new ArrayList<Location>();
		findPath(cellX[i],xfinal,cellY[i],yfinal,CA);
		//we have a list of locations
		ArrayList<RouteSegment> rs = vnsim.vehicular.routePlan.RouteComputingUtils.mergeRoute(SegmentsRoute);
		return rs;
	}
	
}
